// Global header file

#include <stdio.h>
#include <time.h>

#ifdef LOCAL
#define EXTERN
#else
#define EXTERN extern
#endif


// Some global defines

#define MAXADMINS 5				// Max no admins in gossip
#define QUEUESIZE 10				// Max no queued commands

// How long to wait between retrying connecting to GO (in seconds)
#define CONNECT_RETRY_WAITTIME 2


// Vars
EXTERN struct client {
  int socket;
  int loggedin;
  int relaylogs;
  time_t logintime;
  time_t lasttime;
  char ip[16];
};

EXTERN struct nameban_el {
  char *name;
  struct nameban_el *next;
};


extern int errno;


EXTERN int do_checklogfiles;	// If this is 1, gossip checks for new
										// logfiles. Should be set every now 
										// and then.	
EXTERN int sockets[2];        // Array of sockets
                              // 0 is for connection with GO,
                              // 1 is for listen()
EXTERN struct client clients[MAXADMINS];	// Array holding client-info 
EXTERN struct nameban_el *namebans;


/*******************************************************  version
***************************************************************/

// Defines
#define GOSSIPVERSION "1.1"
#define VERSIONDATE __DATE__
#define VERSIONTIME __TIME__


/******************************************************  config
*************************************************************/

// Defines
#define CONFIGFILE "gossip.conf"
#define CONF_NOTNEEDED 0
#define CONF_NEEDED 1
#define CONF_ELTYPE_STRING 1
#define CONF_ELTYPE_INT 2
#define CONF_ELTYPE_YESNO 3

EXTERN struct conf_template_element {
        int eltype;
        int needed;
        char name[32];
        void *value;
};

#ifdef LOCAL
struct conf_template_element configuration[]={
        { CONF_ELTYPE_STRING, CONF_NEEDED, "adminaddress", NULL },
        { CONF_ELTYPE_INT, CONF_NEEDED, "listenport", NULL },
        { CONF_ELTYPE_INT, CONF_NEEDED, "adminport", NULL },
        { CONF_ELTYPE_STRING, CONF_NEEDED,"adminpass",NULL },
        { CONF_ELTYPE_INT, CONF_NEEDED, "login_timeout", NULL },
        { CONF_ELTYPE_INT, CONF_NEEDED,"inactivity_timeout",NULL },
        { CONF_ELTYPE_INT, CONF_NEEDED, "connects_per_ip", NULL },
		{ CONF_ELTYPE_STRING, CONF_NOTNEEDED, "logdirectory", NULL },
		{ CONF_ELTYPE_STRING, CONF_NOTNEEDED, "banlist", NULL },
		{ CONF_ELTYPE_YESNO, CONF_NEEDED, "logrelay", NULL },
		{ CONF_ELTYPE_STRING, CONF_NOTNEEDED, "logfilename", NULL },
		{ CONF_ELTYPE_INT, CONF_NEEDED, "logmethod", NULL },
		{ CONF_ELTYPE_YESNO, CONF_NOTNEEDED, "debug", NULL },
		{ CONF_ELTYPE_YESNO, CONF_NOTNEEDED, "daemonize", NULL },
        { 0,0,"",NULL },
};
#else
extern struct conf_template_element configuration[];
#endif

EXTERN char *configfile;

/*******************************************************  logger
**************************************************************/

// Defines
#define LOGGER_DEBUG 1
#define LOGGER_INFO 2
#define LOGGER_WARNING 3
#define LOGGER_ERROR 4

#define LOGGER_NONE 0
#define LOGGER_SYSLOG 1
#define LOGGER_FILE 2
#define LOGGER_BOTH 3

// Variables
EXTERN FILE *logfd;


/*******************************************************  queue
***************************************************************/

EXTERN int queue[QUEUESIZE];		// Queue to hold clients 		
EXTERN int queue_start;				// Needed to provide for a queue
EXTERN int queue_end;
