#include <globals.h>
#include <logger.h>
#include <conf.h>

#include <syslog.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <time.h>


// Local variables
int             logger_do_stop = 0;

int
logger_init(void)
{
    if (conf_getint("logmethod") & LOGGER_SYSLOG) {
	openlog("gossip", 0, LOG_DAEMON);
    }

    if (conf_getint("logmethod") & LOGGER_FILE) {
	if ((logfd = fopen((char *)conf_get("logfilename"), "a+")) == NULL) {
	    perror("Could not open logfile for writing");
	    return -1;
	}
    }

    return 0;
}


void
do_log(int severity, const char *fmt, ...)
{
    char           *timestring;
    char            buf[1024];
    time_t          now;
    char            severitystring[12];
    char           *s,
                   *msg;
    char            logstring[512];
    va_list         ap;

    // If logging is stopped, message is debugging but debugging is
    // disabled or logmethod is none, don't log
    if (logger_do_stop || (severity == LOGGER_DEBUG && !(conf_isyes("debug")))
	|| conf_getint("logmethod") == LOGGER_NONE)
	return;

    // Build a logstring from the formatstring and other args
    va_start(ap, fmt);
    vsnprintf(logstring, sizeof(logstring), fmt, ap);
    va_end(ap);

    // Filter some crap
    msg = strdup(logstring);
    for (s = msg; *s; s++) {
	if (*s == '\n' || *s == '\r' || *s == '\f')
	    *s = 0;
	if (*s == '\b')
	    *s = ' ';
    }

    // Write logline to stdout
    printf("%s\n", logstring);

    if (conf_getint("logmethod") & LOGGER_SYSLOG) {
	switch (severity) {
	case LOGGER_DEBUG:
	    syslog(LOG_DEBUG, "%s", msg);
	    break;
	case LOGGER_INFO:
	    syslog(LOG_INFO, "%s", msg);
	    break;
	case LOGGER_WARNING:
	    syslog(LOG_WARNING, "%s", msg);
	    break;
	case LOGGER_ERROR:
	    syslog(LOG_ERR, "%s", msg);
	    break;
	default:
	    do_log(LOGGER_ERROR, "The logger was given a wrong severity-level, this should not happen");
	}
    }

    if (conf_getint("logmethod") & LOGGER_FILE) {
	time(&now);
	timestring = ctime(&now);

	// Filter out the \n ctime leaves behind
	timestring[strlen(timestring) - 1] = '\0';

	switch (severity) {
	case LOGGER_DEBUG:
	    strcpy(severitystring, "DEBUG: ");
	    break;
	case LOGGER_INFO:
	    strcpy(severitystring, "INFO: ");
	    break;
	case LOGGER_WARNING:
	    strcpy(severitystring, "WARNING: ");
	    break;
	case LOGGER_ERROR:
	    strcpy(severitystring, "ERROR: ");
	    break;
	default:
	    do_log(LOGGER_ERROR, "The logger was given a wrong severity-level, this should not happen");
	}

	// Construct string to be written to file
	snprintf(buf, sizeof(buf), "%s %s%s\n", timestring, severitystring, msg);

	if ((fwrite(buf, strlen(buf), 1, logfd)) < strlen(buf)) {
	    if (conf_getint("logmethod") & LOG_SYSLOG)
		syslog(LOG_ERR, "There was an error writing to the logfile %s", (char *)conf_get("logfilename"));
	}
	fflush(logfd);
    }
    // Free malloc()ed mem
    free(msg);
}

void
logger_stop(void)
{
    logger_do_stop = 1;
}
