#include <queue.h>
#include <logger.h>
#include <globals.h>

/*
  The queue works as follows: 
 
  queue_start = element-no of first queue-element in the array 
  queue_end = element-no of last queue-element in the array 

  If queue_start==-1, then queue is empty.  Adding to the queue is done by adding 1 to
  queue_end and putting the new element there. If queue_end bumps to the end of the 
  array, the whole queue is moved to the start of the array, so that the element can be 
  added. Getting an element off of the queue is done by adding 1 to queue_start and 
  returning the first element of the queue (that is: queue[queue_start] before adding
  the 1).  
  
*/


void
init_queue(void)
{
    queue_start = -1;
    queue_end = -1;
}

// Get the first element of the queue
int
queue_top(void)
{
    if (queue_start == -1)
	return -1;
    else
	return queue[queue_start];
}

// Get the first element of the queue, delete it from the queue
int
queue_pop(void)
{
    int             tempint;

    if (queue_start == -1)
	return -1;
    else {
	tempint = queue[queue_start];
	queue_start++;
	if (queue_start > queue_end) {
	    queue_clear();
	}
    }

    return tempint;
}

// Put this element in the queue
int
queue_put(int a)
{
    int             i;

    if ((queue_end - queue_start) == (QUEUESIZE - 1)) {
	do_log(LOGGER_WARNING, "Command-queue full, please raise QUEUESIZE in globals.h and recompile");
	return -1;
    }
    // Is queue empty? If so: 'initialize'
    if (queue_empty()) {
	queue_start = 0;
    }
    queue_end++;
    if (queue_end == QUEUESIZE) {
	// Queue reached end of array, move it a bit, I don't like those
	// rotational queues, I have to think to get the implementation
	// of those right, and which programmer likes thinking?
	for (i = 0; i < QUEUESIZE - 1; i++)
	    queue[i] = queue[i + 1];
	queue_end = queue_end - queue_start;
	queue_start = 0;
    }
    queue[queue_end] = a;
    return 0;			// Success
}

// Is queue empty?
int
queue_empty(void)
{
    if (queue_start == -1)
	return 1;
    else
	return 0;
}

// Make queue empty
void
queue_clear(void)
{
    queue_start = -1;
    queue_end = -1;
}
